/******************************************************************************
*
* Copyright (C) 2006-2017  Brian Berlin
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
*
******************************************************************************
*
* FILENAME: filebrowser.hpp
*
* DESCRIPTION:  This class will show a pop-up for loading / saving files
*
* AUTHORS:  bberlin
*
* CHANGE LOG
*
*  VER     DATE     CHANGED BY   DESCRIPTION OF CHANGE
* ------  --------  ----------   ------------------------------------
* 0.5.0   09/22/06  bberlin      Creation
******************************************************************************/
#ifndef filebrowser_hpp
#define filebrowser_hpp

#include <guichan.hpp>
#include "genericlist.hpp"
#include "guiobject.hpp"
#include "window.hpp"

class FileBrowser : public gcn::GuiWindow,
                    public GuiObject,
                    public gcn::ActionListener,
                    public gcn::KeyListener
{
    public:
        FileBrowser( GuiMain *gMain );
        ~FileBrowser();

        void show ( std::string title, std::string dir, int type );
        void hide ( );
        void setOkText ( std::string text );
        void setCancelPressed ( bool pressed );
        bool getCancelPressed ( );
        void setShowOverwrite ( bool show );
        void setDirOnly ( bool setting );
        void setResult ( std::string result );
        std::string getResult ( );
        void deleteActions ( );
        void fillFileList ( std::string dir, std::string substring );
        char getSeperatorChar ( );
        void action(const gcn::ActionEvent &actionEvent);
        void keyPressed (gcn::KeyEvent &keyEvent);
        void keyReleased (gcn::KeyEvent &keyEvent);

        enum
        {
            EMPTY = 0,
            LOAD,
            SAVE,
            DIR_ONLY,
            SELECT,
            CANCEL
        };

    private:
        std::string mDirText;
        std::string mResult;
        gcn::Button mOk;
        gcn::Button mCancel;
        gcn::Button mUp;
        gcn::TextField mFile;
        gcn::TextField mDir;
        gcn::ListBox mDirListBox;
        gcn::ScrollArea mListScroller;
        GenericList mFileList;
        bool mShowDirOnly;
        bool mCancelPressed;
        bool mShowOverwrite;
        int mType;
        unsigned long mClickTime;
        unsigned long mKeyTime;
        int mKeyLevel;
        int mKeySearchType;
        int mCurrentIndex;
        char mKeySave[4];
};

#endif
