/******************************************************************************
*
* Copyright (C) 2006-2017  Brian Berlin
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
*
******************************************************************************
*
* FILENAME: gui.cpp
*
* DESCRIPTION:  This is the *main* for the GUI
*
* AUTHORS:  bberlin
*
* CHANGE LOG
*
*  VER     DATE     CHANGED BY   DESCRIPTION OF CHANGE
* ------  --------  ----------   ------------------------------------
* 0.5.0   11/03/06  bberlin      Creation
******************************************************************************/
#include <iostream>
#include "guimain.hpp"
#include "shadercompile.hpp"

extern "C" int gui_show_main ( int error, int debug_flag )
{
    try
    {
        GuiMain *guimain = new GuiMain ( error, debug_flag );
        bool exit, load, mode;

        guimain->run();

        exit = guimain->getExit();
        load = guimain->getStateLoad();
        mode = guimain->getDebugMode();

        delete guimain;

        if (exit == true )
            return -1;
        else if ( load == true )
            return 2;
        else if ( mode == true )
            return 0;
        else
            return 1;
    }
    catch (gcn::Exception e)
    {
         std::cout << e.getMessage() << std::endl;
        return 1;
    }
     catch (std::exception e)
    {
       std::cout << "Std exception: " << e.what() << std::endl;
        return 1;
    }
    catch (...)
    {
        std::cout << "Unknown exception" << std::endl;
        return 1;
    }

    return 0;
}

extern "C" int gui_show_shader_compile ( )
{
    try
    {
        ShaderCompile *shadercompile = new ShaderCompile ( );

        shadercompile->run();

        delete shadercompile;

        return 0;
    }
    catch (gcn::Exception e)
    {
         std::cout << e.getMessage() << std::endl;
        return 1;
    }
     catch (std::exception e)
    {
       std::cout << "Std exception: " << e.what() << std::endl;
        return 1;
    }
    catch (...)
    {
        std::cout << "Unknown exception" << std::endl;
        return 1;
    }

    return 0;
}
