/******************************************************************************
*
* Copyright (C) 2006-2017  Brian Berlin
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
*
******************************************************************************
*
* FILENAME: inputcfg.hpp
*
* DESCRIPTION:  This class will show the input configuration
*
* AUTHORS:  bberlin
*
* CHANGE LOG
*
*  VER     DATE     CHANGED BY   DESCRIPTION OF CHANGE
* ------  --------  ----------   ------------------------------------
* 0.5.0   11/28/06  bberlin      Creation
******************************************************************************/
#ifndef inputcfg_hpp
#define inputcfg_hpp

#include <guichan.hpp>
#include "genericlist.hpp"
#include "tabbedwindow.hpp"
#include "guiinputbutton.hpp"
#include "window.hpp"
#include "inputautolistener.hpp"
#include "decoration.hpp"
#include "guiobject.hpp"

extern "C" {
    #include "../interface/input.h"
    #include "../interface/kconfig.h"
    #include "../interface/util.h"
}

class InputCfg : public gcn::GuiWindow,
                 public GuiObject,
                 public gcn::ActionListener
{
    public:
        InputCfg( GuiMain *gMain );
        ~InputCfg();

        void show (); 
        void hide ();

        void initGroupControls ();
        void changeGroup ( std::string profile_name );
        void setMachine ( e_machine_type type );
        void saveSettings ( std::string profile_name );
        void getInput( int player, t_atari_common *ctrl );
        void setInput( int player, t_atari_common *ctrl );
        void action(const gcn::ActionEvent &actionEvent);

        void mousePressed(gcn::MouseEvent& mouseEvent);

    private:
        double setRow( double row );
        double setColumn( double column );
        void showComboInput ( int player, bool show );
        void setComboGuiValues ( int player, t_atari_common *ctrl );
        std::string getKeyTitle(int key);
        void set800KeyDimensions();
        void initProfileList ( );
        void setupInputButton ( GuiInputButton *btn, std::string caption, 
                                 std::string grp, gcn::ActionListener *listener, 
                                 gcn::Window *parent, t_atari_common *ctrl );

        Decoration mProfileDec;
        gcn::DropDown mProfile;
        GenericList mProfileList;
        gcn::TextField mDescriptionBox;
        gcn::Button mRename;
        gcn::Button mSaveAs;

        gcn::Button mOk;
        gcn::Button mCancel;

        gcn::CheckBox mDefault;

        gcn::Label mMouseLabel;
        gcn::DropDown mMouse;
        GenericList mMouseList;

        TabbedWindow mPlayerWindow;
        gcn::Window mKeyboardTab;
        gcn::Window mPlayerTab[4];

        Decoration mSettingsDec[4];
        gcn::Label mSensitivityLabel[4];
        gcn::Slider mSensitivity[4];
        gcn::Label mSensitivityValue[4];
        gcn::Label mDeadzoneLabel[4];
        gcn::Slider mDeadzone[4];
        gcn::Label mDeadzoneValue[4];
        gcn::CheckBox mSimulateAnalog[4];

        Decoration mTypeDec[4];
        gcn::DropDown mPluggedIn[4];
        GenericList mPluggedInList[4];

        Decoration mPCDec[2][5];
        gcn::Label mDeviceLabel[2][5];
        gcn::DropDown mDevice[2][5];
        GenericList mDeviceList[2][5];
        gcn::Label mDeviceNumLabel[2][5];
        gcn::TextField mDeviceNum[2][5];
        gcn::Label mPartTypeLabel[2][5];
        gcn::DropDown mPartType[2][5];
        GenericList mPartTypeList[2][5];
        gcn::Label mPartNumLabel[2][5];
        gcn::TextField mPartNum[2][5];
        gcn::Label mDirectionLabel[2][5];
        gcn::DropDown mDirection[2][5];
        GenericList mDirectionList[2][5];
        gcn::Button mAutoDetect[2][5];
        gcn::CheckBox mComboInput[5];
        InputAutoListener mAutoListener;

        Decoration mStickDec[4];
        Decoration mPaddlesDec[4];
        Decoration mKeypadDec[4];
        Decoration mA5200Dec[4];

        Decoration mKeysDec;
        Decoration mControlsDec;
        GuiInputButton mKey[10][4];
        GuiInputButton mStarKey[4];
        GuiInputButton mPoundKey[4];
        GuiInputButton mStartKey[4];
        GuiInputButton mPauseKey[4];
        GuiInputButton mResetKey[4];
        GuiInputButton mTopButton[4];
        GuiInputButton mBottomButton[4];
        GuiInputButton m800Button[4];
        GuiInputButton m800Key[64];
        GuiInputButton m800SelectKey;
        GuiInputButton m800StartKey;
        GuiInputButton m800OptionKey;
        GuiInputButton m800ShiftKey;
        GuiInputButton m800CtrlKey;
        GuiInputButton m800BrkKey;
        GuiInputButton m800ResetKey;

        GuiInputButton mStickDirection[4][4];
        GuiInputButton mPaddleDirection[4][4];
        gcn::Label mPaddle1Label[4];
        gcn::Label mPaddle2Label[4];

        t_atari_common *mCurrentCtrl[5];

        t_atari_input mInput;
        bool mSettingsDirty;
        bool mKeyboardShowing;
        int mAutoDetectAct;
        int mCurrentGrp;
};

#endif
