/******************************************************************************
*
* Copyright (C) 2007-2017  Brian Berlin
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
*
******************************************************************************
*
* FILENAME: joystickevent.cpp
*
* DESCRIPTION:  My own version of Guichan's Event class to handle joysticks
*
* AUTHORS:  bberlin
*
* CHANGE LOG
*
*  VER     DATE     CHANGED BY   DESCRIPTION OF CHANGE
* ------  --------  ----------   ------------------------------------
* 0.6.0   12/15/07  bberlin      Creation
******************************************************************************/
#include "joystickevent.hpp"

namespace gcn
{
    JoystickEvent::JoystickEvent(Widget* source,
                           bool isShiftPressed,
                           bool isControlPressed,
                           bool isAltPressed,
                           bool isMetaPressed,
                           unsigned int type,
                           int x,
                           int y,
                           int device,
                           int deviceType,
                           int part,
                           int part_type,
                           int value)
            :InputEvent(source,
                        isShiftPressed,
                        isControlPressed,
                        isAltPressed,
                        isMetaPressed),
             mType(type),
             mX(x),
             mY(y),
             mDevice(device),
             mDeviceType(deviceType),
             mPart(part),
             mPartType(part_type),
             mValue(value)
    {

    }

    JoystickEvent::~JoystickEvent()
    {
    }

    int JoystickEvent::getDevice() const
    {
        return mDevice;
    }

    int JoystickEvent::getDeviceType() const
    {
        return mDeviceType;
    }

    int JoystickEvent::getPart() const
    {
        return mPart;
    }

    int JoystickEvent::getType() const
    {
        return mType;
    }

    int JoystickEvent::getPartType() const
    {
        return mPartType;
    }

    int JoystickEvent::getValue() const
    {
        return mValue;
    }

    int JoystickEvent::getX() const
    {
        return mX;
    }

    int JoystickEvent::getY() const
    {
        return mY;
    }
}
