/******************************************************************************
*
* Copyright (C) 2007-2017  Brian Berlin
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
*
******************************************************************************
*
* FILENAME: joystickevent.hpp
*
* DESCRIPTION:  My own version of Guichan's Event class to handle joysticks
*
* AUTHORS:  bberlin
*
* CHANGE LOG
*
*  VER     DATE     CHANGED BY   DESCRIPTION OF CHANGE
* ------  --------  ----------   ------------------------------------
* 0.6.0   12/15/07  bberlin      Creation
******************************************************************************/
#ifndef GCN_JOYSTICKEVENT_HPP
#define GCN_JOYSTICKEVENT_HPP

#include "guichan/inputevent.hpp"
#include "guichan/platform.hpp"

namespace gcn
{

    class Widget;

    /**
     * Joystick event
     * 
     */
    class JoystickEvent: public InputEvent
    {
    public:

        /**
         * Constructor.
         *
         */
       JoystickEvent(Widget* source,
                 bool isShiftPressed,
                 bool isControlPressed,
                 bool isAltPressed,
                 bool isMetaPressed,
                 unsigned int type,
                 int x,
                 int y,
                 int device,
                 int deviceType,
                 int part,
                 int part_type,
                 int value);

        /**
         * Destructor.
         */
        virtual ~JoystickEvent();

        int getType() const;

        int getDevice() const;
        int getDeviceType() const;
        int getPart() const;
        int getPartType() const;
        int getValue() const;
        int getX() const;
        int getY() const;

        /**
         * Enum with joystick event values. Equal to joystick input
         */
        enum
        {
            EMPTY = 0,
            PRESS,
            RELEASE,
            LEFT,
            RIGHT,
            UP,
            DOWN,
            MOTION,
            HAT,
            BALL,
            AXIS,
            BUTTON,
            JOYSTICK,
            CONTROLLER
        };

    protected:
        int mDevice;
        int mDeviceType;
        int mPart;
        int mPartType;
        int mValue;
        int mX;
        int mY;

        unsigned int mType;
    };
}

#endif // end GCN_JOYSTICKEVENT_HPP
