/******************************************************************************
*
* Copyright (C) 2006-2017  Brian Berlin
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
*
******************************************************************************
*
* FILENAME: katmenulistener.cpp
*
* DESCRIPTION:  Class that contains actions for menu items
*
* AUTHORS:  bberlin
*
* CHANGE LOG
*
*  VER     DATE     CHANGED BY   DESCRIPTION OF CHANGE
* ------  --------  ----------   ------------------------------------
* 0.5.0   10/31/06  bberlin      Creation
******************************************************************************/
#include "filebrowser.hpp"
#include "guimain.hpp"

extern "C" {
    #include "../core/console.h"
    #include "../interface/kconfig.h"
    #include "../interface/states.h"
}

void KatMenuListener::action(const gcn::ActionEvent &actionEvent)
{
    t_config * p_config = config_get_ptr();
    struct atari *p_console = console_get_ptr();

    if ( actionEvent.getId() == "LoadBrowser" )
    {
        if (mGuiMain->getFileBrowser()->getCancelPressed() == false ) {
            strcpy ( p_config->rom_file, mGuiMain->getFileBrowser()->getResult().c_str() );
            mGuiMain->getFileBrowser()->hide();
            mGuiMain->launch( 0, 0 );
        }
    }
    else if ( actionEvent.getId() == "SaveState" )
    {
        if ( mGuiMain->getProfilePopup()->getCancelPressed() == false ) {
            if ( save_state ( mGuiMain->getProfilePopup()->getProfile().c_str(), p_console ) )
                mGuiMain->getInfoPopup()->show( "Error Saving State" );
        }
    }
    else if ( actionEvent.getId() == "LoadState" )
    {
        if ( mGuiMain->getProfilePopup()->getCancelPressed() == false ) {
            strcpy ( p_config->state, mGuiMain->getProfilePopup()->getProfile().c_str() );
            mGuiMain->getProfilePopup()->hide();
            if ( recall_state ( p_config->state, console_get_ptr() ) ) {
                mGuiMain->getInfoPopup()->deleteActions();
                mGuiMain->getInfoPopup()->show( "Error Loading State" );
                return;
            }
            mGuiMain->launch( 1, 0 );
        }
    }
}
