/******************************************************************************
*
* Copyright (C) 2006-2017  Brian Berlin
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
*
******************************************************************************
*
* FILENAME: palettecfg.hpp
*
* DESCRIPTION:  This class will show the palette configuration
*
* AUTHORS:  bberlin
*
* CHANGE LOG
*
*  VER     DATE     CHANGED BY   DESCRIPTION OF CHANGE
* ------  --------  ----------   ------------------------------------
* 0.5.0   10/24/06  bberlin      Creation
******************************************************************************/
#ifndef palettecfg_hpp
#define palettecfg_hpp

#include <guichan.hpp>
#include "guiobject.hpp"
#include "window.hpp"
#include "genericlist.hpp"
#include "decoration.hpp"

class PaletteCfg : public gcn::GuiWindow,
                   public GuiObject,
                   public gcn::ActionListener
{
    public:
        PaletteCfg( GuiMain *gMain );
        ~PaletteCfg();

        void show (); 
        void hide ();

        void initProfileList ();
        void initGroupControls ();
        void changeGroup ( std::string profile_name );
        void saveSettings ( std::string profile_name );

        void action(const gcn::ActionEvent &actionEvent);
        void mousePressed(gcn::MouseEvent& mouseEvent);

    private:
        gcn::DropDown mProfile;
        GenericList mProfileList;
        gcn::Label mDescriptionLabel;
        gcn::TextField mDescriptionBox;
        gcn::Button mRename;
        gcn::Button mSaveAs;
        Decoration mProfileDec;

        gcn::Button mOk;
        gcn::Button mCancel;

        Decoration mPalDec;
        gcn::Label mIndexLabel;
        gcn::TextField mIndex;
        gcn::Label mValueLabel;
        gcn::TextField mValue;

        std::string mCurrentName;
        bool mSettingsDirty;
        int mCurrentIndex;
        int mPal[256];
};

#endif
