/******************************************************************************
*
* Copyright (C) 2017  Brian Berlin
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
*
******************************************************************************
*
* FILENAME: shadercfg.cpp
*
* DESCRIPTION:  This class will show the shader configuration
*
* AUTHORS:  bberlin
*
* CHANGE LOG
*
*  VER     DATE     CHANGED BY   DESCRIPTION OF CHANGE
* ------  --------  ----------   ------------------------------------
* 0.8.0   04/15/17  bberlin      Creation
******************************************************************************/
#include <math.h>
extern "C" {
    #include "../interface/video.h"
    #include "../interface/shader.h"
    #include "../interface/db_if.h"

    extern t_atari_video g_video;
}

#include "shadercfg.hpp"
#include "guimain.hpp"

t_master_shader g_master1;

ShaderCfg::ShaderCfg ( GuiMain *gMain ) : GuiObject(gMain)
{
    setVisible ( false );
    setCaption ("Shader Settings");

    /*
     * Setup all of our widgets
     */
    mProfileDec.setCaption ( "Shader" );
    mRename.setCaption ("Rename");
    mSaveAs.setCaption ("Save As");
    mDelete.setCaption ("Delete");
    mReset.setCaption ("Set to Defaults");
    mProfile.setListModel ( &mProfileList );
    mShader.setListModel ( &mShaderList );

    mOk.setCaption ("Ok");
    mCancel.setCaption ("Cancel");

    mPalDec.setCaption ( "" );
    mParamLabel.setCaption ("Parameters for File");
    mParamScroller.setContent ( &mParamContainer );
    shader_init_master ( &g_master1 );

    /*
     * Create Callbacks and assign
     */
    addActionListener ( this );

    mProfile.addActionListener ( this );
    mShader.addActionListener ( this );
    mRename.addActionListener ( this );
    mSaveAs.addActionListener ( this );
    mDelete.addActionListener ( this );
    mReset.addActionListener ( this );
    mOk.addActionListener ( this );
    mCancel.addActionListener ( this );

    mRename.setActionEventId ( "mRename" );
    mSaveAs.setActionEventId ( "mSaveAs" );
    mDelete.setActionEventId ( "mDelete" );
    mReset.setActionEventId ( "mReset" );
    mOk.setActionEventId ( "mOk" );
    mCancel.setActionEventId ( "mCancel" );
    mProfile.setActionEventId ( "mProfile" );
    mShader.setActionEventId ( "mShader" );

    /*
     * Size and place our widgets
     */
    add ( &mProfileDec );
    add ( &mPalDec );
    add ( &mProfile );
    add ( &mShader );
    add ( &mRename );
    add ( &mSaveAs );
    add ( &mDelete );
    add ( &mReset );

    add ( &mOk );
    add ( &mCancel );

    add ( &mParamScroller );
    add ( &mParamLabel );

    setDimension ( mGuiMain->getRectangle(3,2,73,24) );

    mProfile.setDimension ( mGuiMain->getRectangle(1,1.2,40,4) );
    mRename.setDimension ( mGuiMain->getRectangle(43,1.2,3,1) );
    mSaveAs.setDimension ( mGuiMain->getRectangle(53.5,1.2,3,1) );
    mDelete.setDimension ( mGuiMain->getRectangle(63.5,1.2,3,1) );
    mProfileDec.setDimension ( mGuiMain->getRectangle(0.5,0.0,71,3.5) );

    mOk.setDimension ( mGuiMain->getRectangle(30,21,3,1) );
    mCancel.setDimension ( mGuiMain->getRectangle(37,21,6,1) );

    mPalDec.setDimension ( mGuiMain->getRectangle(0.5,3.5,71,17.0) );
    mParamLabel.setDimension ( mGuiMain->getRectangle(1,4.5,20,1) );
    mShader.setDimension ( mGuiMain->getRectangle(1,6.0,69,1) );
    mReset.setDimension ( mGuiMain->getRectangle(55,4.3,10,1) );
    mParamScroller.setDimension ( mGuiMain->getRectangle(1,7.4,69,13.5) );
    mParamContainer.setDimension ( mGuiMain->getRectangle(0,0,71,15.5) );

    mProfile.adjustHeight();
    mShader.adjustHeight();
    mRename.adjustSize();
    mSaveAs.adjustSize();
    mDelete.adjustSize();
    mReset.adjustSize();
    mOk.adjustSize();
    mCancel.adjustSize();

    mParamLabel.adjustSize();

    /*
     * Initialize the list
     */
    initProfileList();
    mCurrentName = "";
}

ShaderCfg::~ShaderCfg ()
{
    unsigned int i;

    // Remove old widgets
    for ( i = 0; i < mParams.size(); ++i ) {
        delete mParams.at(i);
        delete mParamsVals.at(i);
        delete mParamsDesc.at(i);
    }

    shader_free_master_contents ( &g_master1 );
}

void ShaderCfg::show ( )
{
    std::string profile_name;

    mSettingsDirty = false;

    initProfileList();
    profile_name = g_video.shader;

    if ( getParent () )
        getParent()->moveToTop (this);

    changeGroup ( profile_name );
    setVisible ( true );
}

void ShaderCfg::hide ( )
{
    setVisible ( false );
    if ( getParent () )
        getParent()->moveToBottom (this);
}

void ShaderCfg::initProfileList ( )
{
    struct generic_node *nodeptr = 0;

    mProfileList.deleteAll();
    db_if_get_profile_names ( "SELECT Name FROM MasterShader", &nodeptr );
    mProfileList.setList ( nodeptr );
}

void ShaderCfg::initShaderList ( )
{
    int i;

    mShaderList.deleteAll();
    mShaderList.setSortMethod ( GLIST_SORT_NONE );
    for ( i = 0; i < g_master1.num_shaders; ++i ) {
        mShaderList.addElement ( g_master1.shaders[i].file, 0, 0 );
    }
}

void ShaderCfg::saveSettings ( std::string profile_name )
{
    int i;
    t_param *tmp_param;

    // Update current parameter values
    for ( i = 0; i < g_master1.shaders[mCurrentShader].num_params; ++i ) {
        tmp_param = &g_master1.shaders[mCurrentShader].params[i];
        g_master1.shaders[mCurrentShader].params[i].val = (GLfloat)mParams.at(i)->getValue();
    }

    // Save the profile and reset the list
    if ( !shader_save_profile ( (char *)profile_name.c_str(), &g_master1 ) ) {

        initProfileList();
        mProfile.setSelected ( mProfileList.getIndexFromLabel(profile_name.c_str()));
        if ( !strcmp ( (char*)profile_name.c_str(), shader_get_master()->name ) ) {
            shader_get_master()->profile_dirty = 1;
        }
    }
}

void ShaderCfg::deleteSettings ( std::string profile_name )
{
    char tmp_string[257];

    sprintf ( tmp_string, "DELETE FROM MasterShader WHERE Name='%s'", 
              profile_name.c_str() );
    db_if_exec_sql ( tmp_string, NULL, NULL );
    sprintf ( tmp_string, "DELETE FROM ShaderUniforms WHERE Master='%s'", 
              profile_name.c_str() );
    db_if_exec_sql ( tmp_string, NULL, NULL );
    sprintf ( tmp_string, "UPDATE Video SET Shader='None' WHERE Shader='%s'", 
              profile_name.c_str() );
    db_if_exec_sql ( tmp_string, NULL, NULL );
    if ( strcmp(g_video.shader, profile_name.c_str()) == 0) 
        strcpy ( g_video.shader, "None" );
}

void ShaderCfg::changeGroup ( std::string profile_name )
{
    /*
     * Save current settings, if necessary
     */
    if ( mSettingsDirty == true ) {
        saveSettings( mProfileList.getLabelFromIndex(mProfile.getSelected()) );
    }

    mSettingsDirty = false;
    shader_free_master_contents ( &g_master1 );

    /*
     * Get values for this Group
     */
    if ( shader_load_profile ( profile_name.c_str(), &g_master1 ) ) {

        mGuiMain->showError(ERR_PROFILE_LOAD);

        profile_name = "None";
    }

    shader_open_master ( (char *)profile_name.c_str(), &g_master1);

    mCurrentName = profile_name;
    mProfile.setSelected ( mProfileList.getIndexFromLabel(profile_name.c_str()) );

    /*
     * Set shader list 
     */
    initShaderList ( );
    mShader.setSelected ( 0 );
    changeShader( 0 );
}

void ShaderCfg::changeShader ( int shader_index )
{
    unsigned int i;
    t_param *tmp_param;

    /*
     * Save current settings, if necessary
     */
    if ( mSettingsDirty == true ) {
        saveSettings( mShaderList.getLabelFromIndex(mCurrentShader) );
    }

    mSettingsDirty = false;

    /*
     * Get values for this Group
     */

    mCurrentShader = shader_index;
    mShader.setSelected ( shader_index );

    /*
     * Set shader values 
     */

    // Remove old widgets
    for ( i = 0; i < mParams.size(); ++i ) {
        delete mParams.at(i);
        delete mParamsVals.at(i);
        delete mParamsDesc.at(i);
    }

    mParams.clear();
    mParamsDesc.clear();
    mParamsVals.clear();

    if (!g_master1.shaders)
        return;

    for ( i = 0; i < (unsigned int)g_master1.shaders[shader_index].num_params; ++i ) {
        tmp_param = &g_master1.shaders[shader_index].params[i];
        mParams.push_back ( new gcn::Slider(tmp_param->min, tmp_param->max) );
        mParamsDesc.push_back ( new gcn::Label(tmp_param->description) );
        mParamsVals.push_back ( new gcn::Label() );
        mParams.at(i)->setValue(tmp_param->val);
        mParamContainer.add(mParams.at(i));
        mParamContainer.add(mParamsDesc.at(i));
        mParamContainer.add(mParamsVals.at(i));
        mParamsDesc.at(i)->setDimension ( mGuiMain->getRectangle(1.5,0.8+i*1.5,36,1) );
        mParams.at(i)->setDimension ( mGuiMain->getRectangle(38,0.8+i*1.5,12,1) );
        mParamsVals.at(i)->setDimension ( mGuiMain->getRectangle(52,0.8+i*1.5,6,1) );
        mParams.at(i)->addActionListener(this);
        mParams.at(i)->setActionEventId("mParam");
        mParams.at(i)->setStepLength(tmp_param->step);
    }

    setParamCaptions ();

    if ( i < 10 ) i = 10;
    mParamContainer.setDimension ( mGuiMain->getRectangle(0,0,71,i*1.5+1) );
}

void ShaderCfg::setParamCaptions()
{
    char tmp_string[100];
    char format[16];
    t_param *tmp_param;
    int i;
    double new_value;

    if (!g_master1.shaders)
        return;

    for ( i = 0; i < g_master1.shaders[mCurrentShader].num_params; ++i ) {
        tmp_param = &g_master1.shaders[mCurrentShader].params[i];
        if ( tmp_param->step < 0.0099 )
            strcpy ( format, "%+#05.3lf" );
        else if ( tmp_param->step < 0.099 )
            strcpy ( format, "%+#04.2lf" );
        else if ( tmp_param->step < 0.999 )
            strcpy ( format, "%+#03.1lf" );
        else
            strcpy ( format, "%+0.0lf" );
        new_value = round(((mParams.at(i)->getValue() - tmp_param->def) / 
                                                        tmp_param->step));
        new_value = (new_value * tmp_param->step) + tmp_param->def;
        mParams.at(i)->setValue(new_value);
        sprintf ( tmp_string, format, mParams.at(i)->getValue() );
        mParamsVals.at(i)->setCaption( tmp_string );
        mParamsVals.at(i)->adjustSize();
    }

}

void ShaderCfg::action(const gcn::ActionEvent &actionEvent)
{
    std::string new_name;
    char tmp_string[257];
    t_param *tmp_param;
    int i;

    if ( actionEvent.getId() == "mOk" ) {
        if ( mSettingsDirty == true ) {
            saveSettings( mProfileList.getLabelFromIndex(mProfile.getSelected()) );
        }
        hide();
    }
    else if ( actionEvent.getId() == "mCancel" ) {
        hide();
    }
    else if ( actionEvent.getId() == "mProfile" ) {
        if ( mSettingsDirty == true ) {
            saveSettings( mCurrentName );
        }
        changeGroup ( mProfileList.getLabelFromIndex(mProfile.getSelected()) );
    }
    else if ( actionEvent.getId() == "mShader" ) {
        if ( mSettingsDirty == true ) {
            saveSettings( mCurrentName );
        }
        changeShader ( mShader.getSelected() );
    }
    else if ( actionEvent.getId() == "mRename" ) {
        if ( !mProfileList.getNumberOfItems() ) 
        {
            mGuiMain->getInfoPopup()->show("No Existing Profiles.  No Action Taken");
            return;
        }
        mGuiMain->getMessagePopup()->deleteActions();
        mGuiMain->getMessagePopup()->addActionListener(this);
        mGuiMain->getMessagePopup()->setActionEventId("Rename");
        mGuiMain->getMessagePopup()->setInput(mProfileList.getLabelFromIndex(mProfile.getSelected()));
        mGuiMain->getMessagePopup()->show("Enter Profile Name", "Ok", true, 
                                             "Cancel", true, "", false, true);
    }
    else if ( actionEvent.getId() == "Rename" ) {
        mGuiMain->getMessagePopup()->hide();
        if ( mGuiMain->getMessagePopup()->getButton() == 0 ) {
            new_name = mGuiMain->getMessagePopup()->getInput();
            if ( mProfileList.getIndexFromLabel(new_name.c_str()) < mProfileList.getNumberOfItems() ) {
                mGuiMain->getInfoPopup()->show("Profile Exists.  No Action Taken");
            }
            else {
                sprintf ( tmp_string, "UPDATE MasterShader SET Name='%s' WHERE Name='%s'", 
                          new_name.c_str(), mProfileList.getLabelFromIndex(mProfile.getSelected()) );
                db_if_exec_sql ( tmp_string, NULL, NULL );
                sprintf ( tmp_string, "UPDATE ShaderUniforms SET Master='%s' WHERE Master='%s'", 
                          new_name.c_str(), mProfileList.getLabelFromIndex(mProfile.getSelected()) );
                db_if_exec_sql ( tmp_string, NULL, NULL );
                if ( strcmp(g_video.shader, mProfileList.getLabelFromIndex(mProfile.getSelected())) == 0) 
                    strcpy ( g_video.shader, new_name.c_str() );
                initProfileList();
                changeGroup (new_name );
            }
        }
    }
    else if ( actionEvent.getId() == "mSaveAs" ) {
        mGuiMain->getProfilePopup()->deleteActions();
        mGuiMain->getProfilePopup()->addActionListener(this);
        mGuiMain->getProfilePopup()->setActionEventId("SaveBrowser");
        mGuiMain->getProfilePopup()->show( "Choose Profile or Create New", "SELECT Name FROM MasterShader", 
                             mProfileList.getLabelFromIndex(mProfile.getSelected()), true );
    }
    else if ( actionEvent.getId() == "SaveBrowser" ) {
        if ( mGuiMain->getProfilePopup()->getCancelPressed() == false ) {
            deleteSettings ( mGuiMain->getProfilePopup()->getProfile() );
            saveSettings ( mGuiMain->getProfilePopup()->getProfile() );
            mSettingsDirty = false;
            initProfileList();
            changeGroup ( mGuiMain->getProfilePopup()->getProfile() );
        }
    }
    else if ( actionEvent.getId() == "mReset" ) {
        for ( i = 0; i < g_master1.shaders[mCurrentShader].num_params; ++i ) {
            tmp_param = &g_master1.shaders[mCurrentShader].params[i];
            tmp_param->val = tmp_param->def;
            mParams.at(i)->setValue(tmp_param->val);
        }
        mSettingsDirty = true;
        setParamCaptions();
    }
    else if ( actionEvent.getId() == "mParam" ) {
        setParamCaptions();
        mSettingsDirty = true;
    }
    else if ( actionEvent.getId() == "mDelete" ) {
        if ( !mProfileList.getNumberOfItems() ) 
        {
            mGuiMain->getInfoPopup()->show("No Existing Shaders.  No Action Taken");
            return;
        }
        if ( !strcmp(mProfileList.getLabelFromIndex(mProfile.getSelected()), "None")  ) 
        {
            mGuiMain->getInfoPopup()->show("Cannot Delete Default Shader.  No Action Taken");
            return;
        }
        mGuiMain->getMessagePopup()->deleteActions();
        mGuiMain->getMessagePopup()->addActionListener(this);
        mGuiMain->getMessagePopup()->setActionEventId("Delete");
        mGuiMain->getMessagePopup()->show("Delete Shader?", "Ok", true, 
                                             "Cancel", true, "", false, false);
    }
    else if ( actionEvent.getId() == "Delete" ) {
        mGuiMain->getMessagePopup()->hide();
        if ( mGuiMain->getMessagePopup()->getButton() == 0 ) {
            deleteSettings(mProfileList.getLabelFromIndex(mProfile.getSelected()));
            initProfileList();
            changeGroup ( "None" );
        }
    }
    else if ( actionEvent.getId() == "mDirty" ) {
        mSettingsDirty = true;
    }
}

void ShaderCfg::mousePressed(gcn::MouseEvent& mouseEvent)
{
    bool comp_has_mouse = false;

    if ((comp_has_mouse == true) && mouseEvent.getButton() == gcn::MouseInput::LEFT)
    {
        mSettingsDirty = true;
    }

    gcn::Window::mousePressed ( mouseEvent );
}
