/******************************************************************************
*
* Copyright (C) 2017  Brian Berlin
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
*
******************************************************************************
*
* FILENAME: shadercfg.hpp
*
* DESCRIPTION:  This class will show the shader configuration
*
* AUTHORS:  bberlin
*
* CHANGE LOG
*
*  VER     DATE     CHANGED BY   DESCRIPTION OF CHANGE
* ------  --------  ----------   ------------------------------------
* 0.8.0   04/15/17  bberlin      Creation
******************************************************************************/
#ifndef shadercfg_hpp
#define shadercfg_hpp

#include <guichan.hpp>
#include "guiobject.hpp"
#include "window.hpp"
#include "genericlist.hpp"
#include "decoration.hpp"

class ShaderCfg : public gcn::GuiWindow,
                   public GuiObject,
                   public gcn::ActionListener
{
    public:
        ShaderCfg( GuiMain *gMain );
        ~ShaderCfg();

        void show (); 
        void hide ();

        void initProfileList ();
        void initShaderList ( );
        void initGroupControls ();
        void changeGroup ( std::string profile_name );
        void changeShader ( int shader_index );
        void saveSettings ( std::string profile_name );
        void deleteSettings ( std::string profile_name );
        void setParamCaptions ();

        void action(const gcn::ActionEvent &actionEvent);
        void mousePressed(gcn::MouseEvent& mouseEvent);

    private:
        gcn::DropDown mProfile;
        GenericList mProfileList;
        gcn::DropDown mShader;
        GenericList mShaderList;
        gcn::Button mRename;
        gcn::Button mSaveAs;
        gcn::Button mDelete;
        gcn::Button mReset;
        Decoration mProfileDec;

        gcn::Button mOk;
        gcn::Button mCancel;

        Decoration mPalDec;
        gcn::Label mParamLabel;
        gcn::ScrollArea mParamScroller;
        gcn::Container mParamContainer;
        std::vector<gcn::Slider *> mParams;
        std::vector<gcn::Label *> mParamsDesc;
        std::vector<gcn::Label *> mParamsVals;

        std::string mCurrentName;
        int mCurrentShader;
        bool mSettingsDirty;
};

#endif
