/******************************************************************************
*
* Copyright (C) 2006-2017  Brian Berlin
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
*
******************************************************************************
*
* FILENAME: soundcfg.hpp
*
* DESCRIPTION:  This class will show the sound configuration
*
* AUTHORS:  bberlin
*
* CHANGE LOG
*
*  VER     DATE     CHANGED BY   DESCRIPTION OF CHANGE
* ------  --------  ----------   ------------------------------------
* 0.5.0   10/24/06  bberlin      Creation
******************************************************************************/
#ifndef soundcfg_hpp
#define soundcfg_hpp

#include <guichan.hpp>
#include "guiobject.hpp"
#include "window.hpp"
#include "genericlist.hpp"
#include "decoration.hpp"

extern "C" {
    #include "../interface/sound.h"
}

class SoundCfg : public gcn::GuiWindow,
                 public GuiObject,
                 public gcn::ActionListener
{
    public:
        SoundCfg( GuiMain *gMain );
        ~SoundCfg();

        void show (); 
        void hide ();

        void initProfileList ();
        void initGroupControls ();
        void changeGroup ( std::string profile_name );
        void setSoundCaptions ();
        void setFilterEnabled ( bool enabled );
        void saveSettings ( std::string profile_name );

        void action(const gcn::ActionEvent &actionEvent);
        void mousePressed(gcn::MouseEvent& mouseEvent);

    private:
        gcn::DropDown mProfile;
        GenericList mProfileList;
        gcn::Label mDescriptionLabel;
        gcn::TextField mDescriptionBox;
        gcn::Button mRename;
        gcn::Button mSaveAs;
        Decoration mProfileDec;

        gcn::Button mOk;
        gcn::Button mCancel;

        Decoration mSoundDec;
        gcn::CheckBox mDefault;
        gcn::CheckBox mSoundEnabled;
        gcn::CheckBox mUseCallback;
        gcn::CheckBox mFilterEnabled;
        gcn::Label mFreqLabel;
        gcn::DropDown mFreq;
        GenericList mFreqList;
        gcn::Label mSamplesLabel;
        gcn::DropDown mSamples;
        GenericList mSamplesList;
        gcn::Label mBufferLabel;
        gcn::Slider mBuffer;
        gcn::Label mBufferValue;

        gcn::Slider mBass;
        gcn::Slider mTreble;

        gcn::Label mBassLabel;
        gcn::Label mTrebleLabel;

        gcn::Label mBassValue;
        gcn::Label mTrebleValue;

        t_atari_sound mSound;

        bool mSettingsDirty;
        int mCurrentGrp;
};

#endif
