/******************************************************************************
*
* Copyright (C) 2006-2017  Brian Berlin
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
*
******************************************************************************
*
* FILENAME: userinterfacecfg.cpp
*
* DESCRIPTION:  This class will show the UI configuration
*
* AUTHORS:  bberlin
*
* CHANGE LOG
*
*  VER     DATE     CHANGED BY   DESCRIPTION OF CHANGE
* ------  --------  ----------   ------------------------------------
* 0.5.0   12/19/06  bberlin      Creation
* 0.8.0   11/24/17  bberlin      Additions/Changes for combo inputs
******************************************************************************/
#include "userinterfacecfg.hpp"
#include "guimain.hpp"

extern "C" {
    #include "../interface/sdl_if.h"
    #include "../interface/db_if.h"

    extern t_ui g_ui;
}

UserInterfaceCfg::UserInterfaceCfg ( GuiMain *gMain ) : GuiObject(gMain),
    mDescriptionBox(" ")
{
    int i,j;
    char tmp_string[30];

    setVisible ( false );
    setCaption ("User Interface Settings");

    /*
     * Setup all of our widgets
     */
    mProfile.setListModel ( &mProfileList );
    mDefault.setCaption ("Use as Default");
    mRename.setCaption ("Rename");
    mSaveAs.setCaption ("SaveAs");
    mDescriptionLabel.setCaption ("Description");
    mProfileDec.setCaption ( "Profile" );

    mOk.setCaption ("Ok");
    mCancel.setCaption ("Cancel");

    mStartDec.setCaption ( "Start Options" );
    mStartGui.setCaption ( "Start Program with GUI" );
    mStartLauncher.setCaption ( "Start GUI with Launcher" );

    for ( j = 0; j < 2; ++j ) {
        if ( j == 0 )
            mPCDec[j].setCaption( "PC Input" );
        else 
            mPCDec[j].setCaption( "Combo First Action" );
        mDeviceLabel[j].setCaption( "Device" );
        mDevice[j].setListModel( &mDeviceList[j] );
        mDeviceNumLabel[j].setCaption ( "Device Num" );
        mPartTypeLabel[j].setCaption ( "Part Type" );
        mPartType[j].setListModel( &mPartTypeList[j] );
        mPartNumLabel[j].setCaption ( "Part Num" );
        mDirectionLabel[j].setCaption ( "Direction" );
        mDirection[j].setListModel( &mDirectionList[j] );
        mAutoDetect[j].setCaption( "Auto-Detect" );
    }
    mComboInput.setCaption( "Combo Input" );

    mUIDec.setCaption ( "" );

    /*
     * Create Callbacks and assign
     */
    addActionListener ( this );

    mProfile.addActionListener ( this );
    mDescriptionBox.addMouseListener ( this );
    mRename.addActionListener ( this );
    mSaveAs.addActionListener ( this );
    mDefault.addActionListener ( this );

    mOk.addActionListener ( this );
    mCancel.addActionListener ( this );

    mProfile.setActionEventId ( "mProfile" );
    mRename.setActionEventId ( "mRename" );
    mSaveAs.setActionEventId ( "mSaveAs" );
    mDefault.setActionEventId ( "mDirty" );

    mOk.setActionEventId ( "mOk" );
    mCancel.setActionEventId ( "mCancel" );

    mStartGui.addActionListener ( this );
    mStartLauncher.addActionListener ( this );
    mStartGui.setActionEventId ( "mDirty" );
    mStartLauncher.setActionEventId ( "mDirty" );

    mDescriptionBox.addFocusListener ( mGuiMain->getTextFocusListener() );

    for ( j = 0; j < 2; ++j ) {
        mDevice[j].addActionListener ( this );
        mDirection[j].addActionListener ( this );
        mPartType[j].addActionListener ( this );
        mAutoDetect[j].addActionListener ( this );

        mDevice[j].setActionEventId ( "mDirty" );
        mDirection[j].setActionEventId ( "mDirty" );
        mPartType[j].setActionEventId ( "mDirty" );
        sprintf ( tmp_string, "mAutoDetect%d", j );
        mAutoDetect[j].setActionEventId ( tmp_string );

        mDeviceNum[j].addMouseListener ( this );
        mPartNum[j].addMouseListener ( this );

        mDeviceNum[j].addFocusListener ( mGuiMain->getTextFocusListener() );
        mPartNum[j].addFocusListener ( mGuiMain->getTextFocusListener() );
    }

    mComboInput.setActionEventId ( "mCombo" );
    mComboInput.addActionListener ( this );

    /*
     * Size and place our widgets
     */
    add ( &mProfileDec );
    add ( &mProfile );
    add ( &mDefault );
    add ( &mRename );
    add ( &mSaveAs );
    add ( &mDescriptionBox );
    add ( &mDescriptionLabel );

    add ( &mOk );
    add ( &mCancel );

    add ( &mStartDec );
    add ( &mStartGui );
    add ( &mStartLauncher );

    for ( j = 0; j < 2; ++j ) {
        add ( &mPCDec[j] );
        add ( &mDeviceLabel[j] );
        add ( &mDevice[j] );
        add ( &mDeviceNumLabel[j] );
        add ( &mDeviceNum[j] );
        add ( &mPartTypeLabel[j] );
        add ( &mPartType[j] );
        add ( &mPartNumLabel[j] );
        add ( &mPartNum[j] );
        add ( &mDirectionLabel[j] );
        add ( &mDirection[j] );
        add ( &mAutoDetect[j] );
    }
    add ( &mComboInput );

    add ( &mUIDec );

    add ( &mStateDec );
    add ( &mStateDec2 );

    setDimension ( mGuiMain->getRectangle(2,2,76,26) );

    mProfile.setDimension ( mGuiMain->getRectangle(1,1.2,54,2) );
    mDescriptionLabel.setDimension ( mGuiMain->getRectangle(1,2.5,54,2) );
    mDescriptionBox.setDimension ( mGuiMain->getRectangle(1,3.5,54,2) );
    mDefault.setDimension ( mGuiMain->getRectangle(57,3.5,14,1) );
    mRename.setDimension ( mGuiMain->getRectangle(57,1.2,3,1) );
    mSaveAs.setDimension ( mGuiMain->getRectangle(65.5,1.2,3,1) );
    mProfileDec.setDimension ( mGuiMain->getRectangle(0.5,0,72.5,5) );

    mOk.setDimension ( mGuiMain->getRectangle(5,22.8,3,1) );
    mCancel.setDimension ( mGuiMain->getRectangle(10,22.8,6,1) );

    //mFontDec.setDimension ( mGuiMain->getRectangle(0.5,5.0,74,3) );
    //mFontFile.setDimension ( mGuiMain->getRectangle(1,6.2,61,1) );
    //mFontBrowse.setDimension ( mGuiMain->getRectangle(64,6,6,1) );

    mStartDec.setDimension ( mGuiMain->getRectangle(0.5,5.0,74,2.5) );
    mStartGui.setDimension ( mGuiMain->getRectangle(11,6,6,1) );
    mStartLauncher.setDimension ( mGuiMain->getRectangle(40,6,6,1) );

    for ( j = 0; j < 2; ++j ) {
        double h = j * 8.7 + 7.2;
        mPCDec[j].setDimension ( mGuiMain->getRectangle(46.5,0.5+h,28.9,8.5) );
        mDeviceLabel[j].setDimension ( mGuiMain->getRectangle(48,1.5+h,6,1) );
        mDevice[j].setDimension ( mGuiMain->getRectangle(60,1.5+h,15,1) );
        mDeviceNumLabel[j].setDimension ( mGuiMain->getRectangle(48,2.5+h,6,1) );
        mDeviceNum[j].setDimension ( mGuiMain->getRectangle(60,2.5+h,15,1) );
        mPartTypeLabel[j].setDimension ( mGuiMain->getRectangle(48,3.5+h,6,1) );
        mPartType[j].setDimension ( mGuiMain->getRectangle(60,3.5+h,15,1) );
        mPartNumLabel[j].setDimension ( mGuiMain->getRectangle(48,4.5+h,6,1) );
        mPartNum[j].setDimension ( mGuiMain->getRectangle(60,4.5+h,15,1) );
        mDirectionLabel[j].setDimension ( mGuiMain->getRectangle(48,5.5+h,6,1) );
        mDirection[j].setDimension ( mGuiMain->getRectangle(60,5.5+h,15,1) );
        mAutoDetect[j].setDimension ( mGuiMain->getRectangle(48,7.0+h,6,1) );
    }
    mComboInput.setDimension ( mGuiMain->getRectangle(61,14.4,6,1) );

    mUIDec.setDimension ( mGuiMain->getRectangle(0.5,8,18,12.5) );
    mExit.setDimension ( mGuiMain->getRectangle(1,9,6,1) );
    mBack.setDimension ( mGuiMain->getRectangle(1,10.7,6,1) );
    mFullScreen.setDimension ( mGuiMain->getRectangle(1,12.4,6,1) );
    mScreenShot.setDimension ( mGuiMain->getRectangle(1,14.1,6,1) );

    mStateDec.setDimension ( mGuiMain->getRectangle(19.5,8,26.2,16.5) );
    for ( i = 0; i < 9; ++i )
    {
        mLoadState[i].setDimension ( mGuiMain->getRectangle(20,9+(i)*1.7,6,1) );
        mSaveState[i].setDimension ( mGuiMain->getRectangle(33,9+(i)*1.7,6,1) );
    }

    setupInputButton ( &mExit, "Exit", "UI", &mUI.keys[UI_KEY_EXIT] );
    setupInputButton ( &mBack, "Back", "UI", &mUI.keys[UI_KEY_BACK] );
    setupInputButton ( &mFullScreen, "Toggle Fullscreen", "UI", &mUI.keys[UI_KEY_FULLSCREEN] );
    setupInputButton ( &mScreenShot, "ScreenShot", "UI", &mUI.keys[UI_KEY_SCREENSHOT] );

    mStateDec.setCaption ( "" );
    mStateDec2.setCaption ( "" );
    for ( i = 1; i < 10; ++i )
    {
        sprintf ( tmp_string, "Load State %d", i );
        setupInputButton ( &mLoadState[i-1], tmp_string, "UI", 
                                &mUI.keys[UI_KEY_LOAD_STATE_1+(i-1)] );
        sprintf ( tmp_string, "Save State %d", i );
        setupInputButton ( &mSaveState[i-1], tmp_string, "UI", 
                                &mUI.keys[UI_KEY_SAVE_STATE_1+(i-1)] );
    }
    mProfile.adjustHeight();
    mDescriptionLabel.adjustSize();
    mDescriptionBox.adjustHeight();
    mRename.adjustSize();
    mSaveAs.adjustSize();
    mDefault.adjustSize();

    mOk.adjustSize();
    mCancel.adjustSize();

    mStartGui.adjustSize();
    mStartLauncher.adjustSize();

    for ( j = 0; j < 2; ++j ) {
        mDeviceLabel[j].adjustSize();
        mDevice[j].adjustHeight();
        mDeviceNumLabel[j].adjustSize();
        mDeviceNum[j].adjustHeight();
        mPartTypeLabel[j].adjustSize();
        mPartType[j].adjustHeight();
        mPartNumLabel[j].adjustSize();
        mPartNum[j].adjustHeight();
        mDirectionLabel[j].adjustSize();
        mDirection[j].adjustHeight();
        mAutoDetect[j].adjustSize();
    }
    mComboInput.adjustSize();

    initGroupControls();
    initProfileList();

    mExit.setSelected ( true );
}

UserInterfaceCfg::~UserInterfaceCfg ()
{

}

void UserInterfaceCfg::setupInputButton (GuiInputButton *btn, std::string caption, 
                                          std::string grp, t_atari_common *ctrl )
{
        btn->setCaption ( caption );
        btn->setGroup ( grp );
        btn->addActionListener ( this );
        btn->setActionEventId ( "mInputButton" );
        add ( btn );
        btn->adjustSize ();
        btn->mCtrl = ctrl;
}

void UserInterfaceCfg::show ( )
{
    mConfig = *(config_get_ptr());
    mUI = g_ui;

    std::string profile_name;

    mSettingsDirty = false;

    profile_name = mUI.name;

    setInput ( mExit.mCtrl );

    mStartGui.setSelected ( mConfig.start_with_gui ? true : false );
    mStartLauncher.setSelected ( mConfig.start_gui_with_launcher ? true : false );

    mSettingsDirty = false;
    mAutoDetectAct = -1;

    if ( getParent () )
        getParent()->moveToTop (this);

    changeGroup ( profile_name );
    setVisible ( true );
}

void UserInterfaceCfg::hide ( )
{
    setVisible ( false );
    if ( getParent () )
        getParent()->moveToBottom (this);
}

void UserInterfaceCfg::initProfileList ( )
{
    struct generic_node *nodeptr = 0;

    mProfileList.deleteAll();
    db_if_get_profile_names ( "SELECT Name FROM UIKeys", &nodeptr );
    mProfileList.setList ( nodeptr );
}

void UserInterfaceCfg::initGroupControls ( )
{
    int j;

    for ( j = 0; j < 2; ++j ) {
        mDeviceList[j].deleteAll();
        mDeviceList[j].addElement ( "Keyboard", DEV_KEYBOARD, 0 );
        mDeviceList[j].addElement ( "Joystick", DEV_JOYSTICK, 0 );
        mDeviceList[j].addElement ( "Mouse", DEV_MOUSE, 0 );

        mPartTypeList[j].deleteAll();
        mPartTypeList[j].addElement ( "D-Pad/Hat", PART_TYPE_HAT, 0 );
        mPartTypeList[j].addElement ( "Axis", PART_TYPE_AXIS, 0 );
        mPartTypeList[j].addElement ( "TrackBall", PART_TYPE_BALL, 0 );
        mPartTypeList[j].addElement ( "Button", PART_TYPE_BUTTON, 0 );
        mPartTypeList[j].addElement ( "Key", PART_TYPE_KEY, 0 );

        mDirectionList[j].deleteAll();
        mDirectionList[j].addElement ( "Left", DIR_LEFT, 0 );
        mDirectionList[j].addElement ( "Right", DIR_RIGHT, 0 );
        mDirectionList[j].addElement ( "Up", DIR_UP, 0 );
        mDirectionList[j].addElement ( "Down", DIR_DOWN, 0 );
        mDirectionList[j].addElement ( "+", DIR_PLUS, 0 );
        mDirectionList[j].addElement ( "-", DIR_MINUS, 0 );
        mDirectionList[j].addElement ( "Press", DIR_PRESS, 0 );
        mDirectionList[j].addElement ( "Release", DIR_RELEASE, 0 );
    }
}

void UserInterfaceCfg::showComboInput ( bool show )
{
    mDeviceLabel[1].setVisible ( show );
    mDevice[1].setVisible ( show );
    mDeviceNumLabel[1].setVisible ( show );
    mDeviceNum[1].setVisible ( show );
    mPartTypeLabel[1].setVisible ( show );
    mPartType[1].setVisible ( show );
    mPartNumLabel[1].setVisible ( show );
    mPartNum[1].setVisible ( show );
    mDirectionLabel[1].setVisible ( show );
    mDirection[1].setVisible ( show );
    mAutoDetect[1].setVisible ( show );
}

void UserInterfaceCfg::setComboGuiValues ( t_atari_common *ctrl )
{

    char tmp_string[20];

    mDevice[1].setSelected ( mDeviceList[1].getIndexFromValue(ctrl->device[1]) );
    sprintf ( tmp_string, "%d", ctrl->device_num[1] );
    mDeviceNum[1].setText( tmp_string );

    mPartType[1].setSelected( mPartTypeList[1].getIndexFromValue( ctrl->part_type[1] ) );

    if ( ctrl->device[1] == DEV_KEYBOARD ) {
        pc_interpret_key_value ( ctrl->part_num[1], tmp_string );
    }
    else {
        sprintf ( tmp_string, "%d", ctrl->part_num[1] );
    }
    mPartNum[1].setText( tmp_string );

    mDirection[1].setSelected ( mDirectionList[1].getIndexFromValue(ctrl->direction[1]) );
}

void UserInterfaceCfg::saveSettings ( std::string profile_name )
{
    t_config *p_config = config_get_ptr();

    // Figure out which inputs are selected and update those
    getInput ( mExit.getMarkedButton()->mCtrl );

    strcpy ( mUI.description, mDescriptionBox.getText().c_str() );

    *(config_get_ptr()) = mConfig;

    if ( !ui_save_profile ( (char *)profile_name.c_str(), &mUI ) ) {

        strcpy ( mUI.name, profile_name.c_str() );

        initProfileList();
        mProfile.setSelected ( mProfileList.getIndexFromLabel(profile_name.c_str()) );

        if ( mDefault.isSelected() == true ) {
            strcpy ( p_config->default_ui_profile, mUI.name );
            ui_load_profile ( profile_name.c_str(), &g_ui );
        }
    }

    p_config->start_with_gui = mStartGui.isSelected() ? 1 : 0;
    p_config->start_gui_with_launcher = mStartLauncher.isSelected() ? 1 : 0;

    config_save ( );
}

void UserInterfaceCfg::changeGroup ( std::string profile_name )
{
    t_config *p_config = config_get_ptr();
    t_ui *p_ui = &mUI;

    /*
     * Save current settings, if necessary
     */
    if ( mSettingsDirty == true ) {
        saveSettings( mProfileList.getLabelFromIndex(mProfile.getSelected()) );
    }

    mSettingsDirty = false;

    /*
     * Get values for this Group
     */
    if ( ui_load_profile ( profile_name.c_str(), p_ui ) ) {
        mGuiMain->showError(ERR_PROFILE_LOAD);

        ui_load_profile ( p_config->default_ui_profile, &mUI );
        profile_name = p_config->default_ui_profile;
    }
    mProfile.setSelected ( mProfileList.getIndexFromLabel(profile_name.c_str()) );

    /*
     * Set values on the GUI
     */
    mDescriptionBox.setText ( p_ui->description );

    if ( !strcmp(profile_name.c_str(), p_config->default_ui_profile) )
        mDefault.setSelected ( true );
    else
        mDefault.setSelected ( false );

    setInput ( mExit.getMarkedButton()->mCtrl );
}

void UserInterfaceCfg::getInput( t_atari_common *ctrl )
{
    char tmp_string[256];

    ctrl->device[0] = (e_dev_type) mDeviceList[0].getValueFromIndex( mDevice[0].getSelected() );

    if ( ctrl->device[0] == DEV_KEYBOARD )
    {
        strcpy ( tmp_string, mDeviceNum[0].getText().c_str() );
        ctrl->device_num[0] = pc_interpret_mod_key(tmp_string);
    }
    else
        ctrl->device_num[0] = atoi(mDeviceNum[0].getText().c_str());

    ctrl->part_type[0] = (e_part_type) mPartTypeList[0].getValueFromIndex( mPartType[0].getSelected() );
    if ( ctrl->device[0] == DEV_KEYBOARD ) {
        strcpy ( tmp_string, mPartNum[0].getText().c_str() );
        ctrl->part_num[0] = pc_interpret_key ( tmp_string );
    }
    else {
        ctrl->part_num[0] = atoi ( mPartNum[0].getText().c_str() );
    }

    ctrl->direction[0] = (e_direction) mDirectionList[0].getValueFromIndex( mDirection[0].getSelected() );

    if ( mComboInput.isSelected() == true )
        ctrl->combo = 1;
    else
        ctrl->combo = 0;

    if ( ctrl->combo ) {

        ctrl->device[1] = (e_dev_type) mDeviceList[1].getValueFromIndex( mDevice[1].getSelected() );
        ctrl->device_num[1] = atoi ( mDeviceNum[1].getText().c_str() );
        ctrl->part_type[1] = (e_part_type) mPartTypeList[1].getValueFromIndex( mPartType[1].getSelected() );
        if ( ctrl->device[1] == DEV_KEYBOARD ) {
            strcpy ( tmp_string, mPartNum[1].getText().c_str() );
            ctrl->part_num[1] = pc_interpret_key ( tmp_string );
        }
        else {
            ctrl->part_num[1] = atoi ( mPartNum[1].getText().c_str() );
        }

        ctrl->direction[1] = (e_direction) mDirectionList[1].getValueFromIndex( mDirection[1].getSelected() );
    }
}

void UserInterfaceCfg::setInput( t_atari_common *ctrl )
{
    char tmp_string[20];

    mCurrentCtrl = ctrl;

    mDevice[0].setSelected ( mDeviceList[0].getIndexFromValue(ctrl->device[0]) );

    if ( ctrl->device[0] == DEV_KEYBOARD )
        pc_interpret_mod_key_value ( ctrl->device_num[0], tmp_string );
    else
        sprintf ( tmp_string, "%d", ctrl->device_num[0] );
    mDeviceNum[0].setText( tmp_string );

    mPartType[0].setSelected( mPartTypeList[0].getIndexFromValue( ctrl->part_type[0] ) );

    if ( ctrl->device[0] == DEV_KEYBOARD )
        pc_interpret_key_value ( ctrl->part_num[0], tmp_string );
    else
        sprintf ( tmp_string, "%d", ctrl->part_num[0] );
    mPartNum[0].setText( tmp_string );

    mDirection[0].setSelected ( mDirectionList[0].getIndexFromValue(ctrl->direction[0]) );

    mComboInput.setSelected( ctrl->combo ? true : false );
    showComboInput ( mComboInput.isSelected() );
    if ( ctrl->combo ) {

        mDevice[1].setSelected ( mDeviceList[1].getIndexFromValue(ctrl->device[1]) );
        sprintf ( tmp_string, "%d", ctrl->device_num[1] );
        mDeviceNum[1].setText( tmp_string );

        mPartType[1].setSelected( mPartTypeList[1].getIndexFromValue( ctrl->part_type[1] ) );

        if ( ctrl->device[1] == DEV_KEYBOARD ) {
            pc_interpret_key_value ( ctrl->part_num[1], tmp_string );
        }
        else {
            sprintf ( tmp_string, "%d", ctrl->part_num[1] );
        }
        mPartNum[1].setText( tmp_string );

        mDirection[1].setSelected ( mDirectionList[1].getIndexFromValue(ctrl->direction[1]) );
    }
}

void UserInterfaceCfg::action(const gcn::ActionEvent &actionEvent)
{
    t_config *p_config = config_get_ptr();
    std::string new_name;
    static int first_axis_event;
    char tmp_string[257];

    if ( actionEvent.getId() == "mOk" ) {
        if ( mSettingsDirty == true ) {
            saveSettings( mProfileList.getLabelFromIndex(mProfile.getSelected()) );
        }
        hide();
    }
    else if ( actionEvent.getId() == "mCancel" ) {
        hide();
    }
    else if ( actionEvent.getId() == "mRename" ) {
        if ( !mProfileList.getNumberOfItems() ) 
        {
            mGuiMain->getInfoPopup()->show("No Existing Profiles.  No Action Taken");
            return;
        }
        mGuiMain->getMessagePopup()->deleteActions();
        mGuiMain->getMessagePopup()->addActionListener(this);
        mGuiMain->getMessagePopup()->setActionEventId("Rename");
        mGuiMain->getMessagePopup()->setInput(mProfileList.getLabelFromIndex(mProfile.getSelected()));
        mGuiMain->getMessagePopup()->show("Enter Profile Name", "Ok", true, 
                                             "Cancel", true, "", false, true);
    }
    else if ( actionEvent.getId() == "Rename" ) {
        mGuiMain->getMessagePopup()->hide();
        if ( mGuiMain->getMessagePopup()->getButton() == 0 ) {
            new_name = mGuiMain->getMessagePopup()->getInput();
            if ( mProfileList.getIndexFromLabel(new_name.c_str()) < mProfileList.getNumberOfItems() ) {
                mGuiMain->getInfoPopup()->show("Profile Exists.  No Action Taken");
            }
            else {
                sprintf ( tmp_string, "UPDATE UIKeys SET Name='%s' WHERE Name='%s'", 
                          new_name.c_str(), mProfileList.getLabelFromIndex(mProfile.getSelected()) );
                db_if_exec_sql ( tmp_string, NULL, NULL );
                initProfileList();
                changeGroup (new_name);
            }
        }
    }
    else if ( actionEvent.getId() == "mSaveAs" ) {
        mGuiMain->getProfilePopup()->deleteActions();
        mGuiMain->getProfilePopup()->addActionListener(this);
        mGuiMain->getProfilePopup()->setActionEventId("SaveBrowser");
        mGuiMain->getProfilePopup()->show( "Choose Profile or Create New", "SELECT Name FROM UIKeys", 
                             mProfileList.getLabelFromIndex(mProfile.getSelected()), true );
    }
    else if ( actionEvent.getId() == "SaveBrowser" ) {
        if ( mGuiMain->getProfilePopup()->getCancelPressed() == false ) {
            saveSettings ( mGuiMain->getProfilePopup()->getProfile() );
            mSettingsDirty = false;
        }
    }
    else if ( actionEvent.getId() == "mDirty" ) {
        mSettingsDirty = true;
    }

    else if ( actionEvent.getId() == "mCombo" ) {
        mSettingsDirty = true;
        showComboInput (mComboInput.isSelected() );
        setComboGuiValues ( mCurrentCtrl );
    }
    else if ( actionEvent.getId() == "mInputButton" ) {
        getInput ( mCurrentCtrl );
        setInput ( ((GuiInputButton *)(actionEvent.getSource()))->mCtrl );
    }

    else if ( actionEvent.getId() == "mAutoDetect0" ||
              actionEvent.getId() == "mAutoDetect1" ) {
        if ( mAutoDetectAct == -1 ) 
        {
            pc_detect_and_open_joysticks ();

            first_axis_event = 1;
            mAutoListener.mCountModKey = false;
            mAutoListener.setWindow ( mGuiMain->getAutoWindow() );

            mGuiMain->getAutoWindow()->deleteAllListeners ( );
            mGuiMain->getAutoWindow()->addActionListener ( this );
            mGuiMain->getAutoWindow()->addKeyListener ( &mAutoListener );
            mGuiMain->getAutoWindow()->addMouseListener ( &mAutoListener );
            mGuiMain->getAutoWindow()->addJoystickListener ( &mAutoListener );

            if ( actionEvent.getId() == "mAutoDetect0" )
                mGuiMain->getAutoWindow()->setActionEventId ( "mAutoWindow0" );
            else
                mGuiMain->getAutoWindow()->setActionEventId ( "mAutoWindow1" );
            mGuiMain->getAutoWindow()->setDimension ( mGuiMain->getRectangle(11,16.5,60,7) );

            mGuiMain->showAutoWindow( "User Action Required", "Perform action to associate with controller part" );
        }
        else {
            mAutoDetectAct--;
        }
    }

    else if ( actionEvent.getId() == "mAutoWindow0" ||
              actionEvent.getId() == "mAutoWindow1" ) {

        int j;
        if ( actionEvent.getId() == "mAutoWindow0" )
            j = 0;
        else 
            j = 1;

        mAutoListener.getInput ( 0, &mCurrentCtrl->device[j], 
                                          &mCurrentCtrl->device_num[j],
                                          &mCurrentCtrl->part_type[j], 
                                          &mCurrentCtrl->part_num[j],
                                          &mCurrentCtrl->direction[j] );
        if ( mCurrentCtrl->device[j] != DEV_JOYSTICK || 
             mCurrentCtrl->part_type[j] != PART_TYPE_AXIS || !first_axis_event )
        {
            mSettingsDirty = true;
            mGuiMain->hideAutoWindow();
            mCurrentCtrl->combo = mComboInput.isSelected();
            setInput ( mCurrentCtrl );
            if ( mCurrentCtrl->device[j] == DEV_MOUSE && 
                 mCurrentCtrl->part_type[j] == PART_TYPE_BUTTON )
                mAutoDetectAct = 0;
        }
        first_axis_event = 0;
    }
}

void UserInterfaceCfg::mousePressed(gcn::MouseEvent &mouseEvent)
{
    bool comp_has_mouse = false;

    if ( &mDeviceNum[0] == mouseEvent.getSource() || 
         &mPartNum[0] == mouseEvent.getSource()  ||
         &mDeviceNum[1] == mouseEvent.getSource() || 
         &mPartNum[1] == mouseEvent.getSource() )
        comp_has_mouse = true;

    if ((comp_has_mouse == true) && mouseEvent.getButton() == gcn::MouseEvent::LEFT)
    {
        mSettingsDirty = true;
    }

    gcn::Window::mousePressed(mouseEvent);
}

