/******************************************************************************
*
* Copyright (C) 2006-2017  Brian Berlin
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
*
******************************************************************************
*
* FILENAME: userinterfacecfg.hpp
*
* DESCRIPTION:  This class will show the UI configuration
*
* AUTHORS:  bberlin
*
* CHANGE LOG
*
*  VER     DATE     CHANGED BY   DESCRIPTION OF CHANGE
* ------  --------  ----------   ------------------------------------
* 0.5.0   12/19/06  bberlin      Creation
******************************************************************************/
#ifndef userinterfacecfg_hpp
#define userinterfacecfg_hpp

#include <guichan.hpp>
#include "genericlist.hpp"
#include "guiinputbutton.hpp"
#include "window.hpp"
#include "inputautolistener.hpp"
#include "decoration.hpp"
#include "guiobject.hpp"

extern "C" {
    #include "../interface/kconfig.h"
    #include "../interface/ui.h"
}

class UserInterfaceCfg : public gcn::GuiWindow,
                         public GuiObject,
                         public gcn::ActionListener
{
    public:
        UserInterfaceCfg( GuiMain *gMain );
        ~UserInterfaceCfg();

        void setupInputButton (GuiInputButton *btn, std::string caption, 
                                 std::string grp, t_atari_common *ctrl );
        void show (); 
        void hide ();
        void initProfileList ( );
        void initGroupControls ();
        void saveSettings ( std::string profile_name );
        void changeGroup ( std::string profile_name );
        void getInput( t_atari_common *ctrl );
        void setInput( t_atari_common *ctrl );
        void action(const gcn::ActionEvent &actionEvent);

        void mousePressed(gcn::MouseEvent &mouseEvent);

    private:
        void showComboInput ( bool show );
        void setComboGuiValues ( t_atari_common *ctrl );

        gcn::DropDown mProfile;
        GenericList mProfileList;
        gcn::CheckBox mDefault;
        gcn::Button mRename;
        gcn::Button mSaveAs;
        gcn::TextField mDescriptionBox;
        gcn::Label mDescriptionLabel;
        Decoration mProfileDec;

        gcn::Button mOk;
        gcn::Button mCancel;

        Decoration mStartDec;
        gcn::CheckBox mStartGui;
        gcn::CheckBox mStartLauncher;

        Decoration mPCDec[2];
        gcn::Label mDeviceLabel[2];
        gcn::DropDown mDevice[2];
        GenericList mDeviceList[2];
        gcn::Label mDeviceNumLabel[2];
        gcn::TextField mDeviceNum[2];
        gcn::Label mPartTypeLabel[2];
        gcn::DropDown mPartType[2];
        GenericList mPartTypeList[2];
        gcn::Label mPartNumLabel[2];
        gcn::TextField mPartNum[2];
        gcn::Label mDirectionLabel[2];
        gcn::DropDown mDirection[2];
        GenericList mDirectionList[2];
        gcn::Button mAutoDetect[2];
        gcn::CheckBox mComboInput;
        InputAutoListener mAutoListener;

        Decoration mUIDec;
        GuiInputButton mExit;
        GuiInputButton mBack;
        GuiInputButton mFullScreen;
        GuiInputButton mScreenShot;

        Decoration mStateDec;
        Decoration mStateDec2;
        GuiInputButton mLoadState[9];
        GuiInputButton mSaveState[9];

        t_atari_common *mCurrentCtrl;

        t_config mConfig;
        t_ui mUI;
        bool mSettingsDirty;
        int mAutoDetectAct;
};

#endif
