/******************************************************************************
*
* Copyright (C) 2006-2017  Brian Berlin
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
*
******************************************************************************
*
* FILENAME: window.cpp
*
* DESCRIPTION:  My own version of Guichan's Window to handle joysticks
*
* AUTHORS:  bberlin
*
* CHANGE LOG
*
*  VER     DATE     CHANGED BY   DESCRIPTION OF CHANGE
* ------  --------  ----------   ------------------------------------
* 0.5.0   09/22/06  bberlin      Creation
******************************************************************************/
#include "window.hpp"

#include "guichan/exception.hpp"
#include "guichan/font.hpp"
#include "guichan/graphics.hpp"
#include "guichan/mouseinput.hpp"
#include "guichan/actionlistener.hpp"

#include "joysticklistener.hpp"

namespace gcn
{
    GuiWindow::GuiWindow()
    {
        addJoystickListener(this);
    }

    GuiWindow::GuiWindow(const std::string& caption)
    {
        addJoystickListener(this);
    }

    void GuiWindow::joyEvent (JoystickEvent &joystickEvent)
    {

    }

    void GuiWindow::generateInputAction ()
    {
        distributeActionEvent();
    }

    void GuiWindow::deleteAllListeners ()
    {
        mJoystickListeners.clear();
        mActionListeners.clear();
        mMouseListeners.clear();
        mKeyListeners.clear();
    }

    void GuiWindow::addJoystickListener(JoystickListener* joystickListener)
    {
        mJoystickListeners.push_back(joystickListener);
    }
 
    void GuiWindow::removeJoystickListener(JoystickListener* joystickListener)
    {
        mJoystickListeners.remove(joystickListener);
    }

    const std::list<JoystickListener*> & GuiWindow::_getJoystickListeners()
    {
        return mJoystickListeners;
    }

    void GuiWindow::drawFrame(Graphics* graphics)
    {
        Color faceColor = getBaseColor();
        Color highlightColor, shadowColor;
        int alpha = getBaseColor().a;
        int width = getWidth() + getFrameSize() * 2 - 1;
        int height = getHeight() + getFrameSize() * 2 - 1;
        highlightColor = faceColor + 0x303030;
        highlightColor.a = alpha;
        shadowColor = faceColor - 0x303030;
        shadowColor.a = alpha;

        unsigned int i;
        for (i = 0; i < getFrameSize(); ++i)
        {
            graphics->setColor(highlightColor);
            graphics->drawLine(i,i, width - i, i);
            graphics->drawLine(i,i + 1, i, height - i - 1);
            graphics->setColor(shadowColor);
            graphics->drawLine(width - i,i + 1, width - i, height - i);
            graphics->drawLine(i,height - i, width - i - 1, height - i);
        }
    }
}
